#include <stdio.h>
#include <stdlib.h>
#include <cv.h>
#include <highgui.h>
#include <math.h>

void quantization(IplImage *img, IplImage *img_q) {
  int height = img->height; // altura
  int width  = img->width;  // largura
  int step   = img->widthStep; // tamanho de uma linha
  uchar *data= img->imageData; // ponteiro para o array de pixels

  int step2   = img_q->widthStep; // tamanho de uma linha
  uchar *data2= img_q->imageData; // ponteiro para o array de pixels
  
  int i,j;

  for (i = 0; i < height; i++) {
      for (j = 0; j < width; j++) {
	   // acessa pixel de imagem com 3 canais de cor:
	   // posicao i,j
	   // canal k
	   // data[i*step+j*3+k] 	   
	   uchar C1 = (data[i*step+j*3+0] & 192)>>2; 
	   uchar C2 = (data[i*step+j*3+1] & 192)>>4; 
	   uchar C3 = (data[i*step+j*3+2] & 192)>>6;	   
	   data2[i*step2+j] = C1 | C2 | C3;
      }    
  } 
}

void GCH(IplImage *img, double *descriptor, int nColor) {
 
  IplImage *img_quant;
  // aloca uma nova imagem do tamanho da original
  // com 8 bits por pixel e 1 canal de cor
  img_quant = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1); 

  quantization(img, img_quant);
  
  int i,j;
  int height = img_quant->height; // altura
  int width  = img_quant->width;  // largura  
  int step   = img_quant->widthStep; // tamanho de uma linha
  uchar *data= img_quant->imageData; // ponteiro para o array de pixels
  
  // percorre cada pixel da imagem incrementando
  // o valor de cada cor no descritor
  for (i = 0; i < height; i++) {
      for (j = 0; j < width; j++) { 
	   descriptor [data[i*step+j]] ++;
      }    
  }
  // normaliza o descritor
  for (i = 0; i < nColor; i++) {
      descriptor[i] = descriptor[i]/ (double)(width*height);    
  }
  
}

// descriptor
// 0..(nColor-1) = Borda
// nColor..(nColor*2-1) = Interior
void BIC(IplImage *img, double *descriptor, int nColor) {
 
  IplImage *img_quant;
  // aloca uma nova imagem do tamanho da original
  // com 8 bits por pixel e 1 canal de cor
  img_quant = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1); 

  quantization(img, img_quant);
  
  int i,j;
  int height = img_quant->height; // altura
  int width  = img_quant->width;  // largura  
  int step   = img_quant->widthStep; // tamanho de uma linha
  uchar *data= img_quant->imageData; // ponteiro para o array de pixels
  
  
  // percorre cada pixel da imagem incrementando
  // o valor de cada cor no descritor
  for (i = 0; i < height; i++) {
      for (j = 0; j < width; j++) { 
	   // armazena pixel atual
	   uchar aux = data[i*step+j];
	   if (i > 0 && j > 0 && i == height && j == width) {
	      // se o pixel tiver cor igual a dos 4-vizinhos
	      if ((aux == data[(i-1)*step+j]) &&
		  (aux == data[(i+1)*step+j]) &&
		  (aux == data[i*step+(j-1)]) &&
		  (aux == data[i*step+(j+1)])) 
	      {
		  // incrementa histograma interior
		  descriptor[aux+nColor] ++;
	      } else {
		  // incrementa histograma borda
		  descriptor[aux] ++;
	      }
	   }
	   else {
	      // incrementa histograma borda
	      descriptor[aux] ++;
	   }
      }    
  }
  // normaliza o descritor
  for (i = 0; i < nColor; i++) {
      descriptor[i] = descriptor[i]/ (double)(width*height);    
  } 
}


double distManhattan(double *p, double *q, int size) {
    int i;
    double dist =0;
    for (i=0; i<size; i++) {
	  dist += fabs(p[i]-q[i]);      
    }
    return dist;  
}